'use strict';

/**
 * 计算器组件
 */
var _ = require('lodash');
var React = require('react');
var YYClass = require('../base/YYClass');
var MathX = require('../../utils/matheval/formula_evaluator');
var PropTypes = YYClass.PropTypes;
var field = YYClass.field;
var YYCalcButton = require('./YYCalcButton');
var YYCalcResult = require('./YYCalcResult');

require('./style/index.less');

function format(str) {
    // 传入值需去除逗号
    var decimal = str.split('.')[1]; //小数部分
    var tempArr = [];
    var revNumArr = str.split('.')[0].split("").reverse(); //倒序
    for (var i = 0; i < revNumArr.length; i++) {
        tempArr.push(revNumArr[i]);
        if ((i + 1) % 3 === 0 && i != revNumArr.length - 1 && revNumArr[i + 1] !== '-') {
            tempArr.push(',');
        }
    }
    var zs = tempArr.reverse().join(''); //整数部分
    return decimal != undefined ? zs + '.' + decimal : zs;
}

//定义META
var uiMeta = {
    name: 'calc',
    fields: [field('isShow', 'false', PropTypes.bool, false, '计算器组件是否显示'), field('onEqual', null, PropTypes.func, false, '点击等于时使结果回显')]
};

var YYCalc = YYClass.create({
    uiMeta: uiMeta,
    getInitialState: function getInitialState() {
        return {
            input_num: '',
            opr: '',
            formula: '',
            result: '0'
        };
    },
    clearState: function clearState() {
        this.setState({
            input_num: '',
            opr: '',
            formula: '',
            result: ''
        });
    },
    initState: function initState(formula) {
        if (formula == undefined) formula = '';
        this.setState({
            input_num: '',
            opr: '',
            formula: formula,
            result: formula || '0'
        });
    },
    onButtonClick: function onButtonClick(type) {
        var _this = this;

        var onEqual = this.props.onEqual;
        var _state = this.state,
            input_num = _state.input_num,
            opr = _state.opr,
            formula = _state.formula;

        switch (type) {
            case 'c':
                {
                    this.setState({
                        input_num: '',
                        opr: '',
                        formula: '',
                        result: '0'
                    });
                    break;
                }
            case 'back':
                {
                    if (input_num) {
                        this.setState({
                            input_num: format(input_num.slice(0, -1).replace(/,/g, ''))
                        });
                        break;
                    }
                    if (opr) {
                        this.setState({
                            opr: ''
                        });
                        break;
                    }
                    if (formula) {
                        this.setState({
                            formula: format(formula.slice(0, -1).replace(/,/g, ''))
                        });
                        break;
                    }
                    break;
                }
            case '=':
                {
                    var reg = /^[\+\-\*\/]|[\+\-\*\/]$/;
                    var foi = formula + opr + input_num;
                    input_num = input_num.replace(/\.$/, '');
                    if (formula + opr + input_num == '') return;
                    if (!reg.test(foi)) this.setState({
                        input_num: format(input_num.replace(/,/g, '')),
                        result: format(_.toString(MathX.eval((formula + opr + input_num).replace(/,/g, ''))))
                    }, function () {
                        if (_.isFunction(onEqual)) {
                            onEqual(_this.state.result);
                        }
                    });
                    break;
                }
            case '+':
            case '-':
            case '*':
            case '/':
                {
                    if (formula + opr + input_num == '') break;
                    if ((formula + opr + input_num).slice(-1) == '+' || (formula + opr + input_num).slice(-1) == '-' || (formula + opr + input_num).slice(-1) == '*' || (formula + opr + input_num).slice(-1) == '/') {
                        type = '';
                    }
                    input_num = input_num.replace(/\.$/, '');
                    this.setState({
                        opr: type,
                        formula: formula + opr + input_num,
                        input_num: ''
                    });
                    break;
                }
            default:
                {
                    if (input_num.indexOf('.') > 0 && type == '.') {
                        type = '';
                    }
                    if (input_num == '' && type == '.') {
                        type = '0.';
                    }
                    if (input_num === '0' && type === '0') {
                        type = '';
                    }
                    this.setState({
                        input_num: format((input_num + type).replace(/,/g, ''))
                    });
                }
        }
    },
    render: function render() {
        var isShow = this.props.isShow;
        var _state2 = this.state,
            formula = _state2.formula,
            result = _state2.result,
            input_num = _state2.input_num,
            opr = _state2.opr;

        var exp = {
            formula: formula + opr + input_num,
            result: result
        };
        var calc = !isShow ? null : React.createElement(
            'div',
            { className: this.className() },
            React.createElement(YYCalcResult, { exp: exp }),
            React.createElement(YYCalcButton, { ref: 'calcButton', onClick: this.onButtonClick })
        );
        return calc;
    }
});
module.exports = YYCalc;