'use strict';

var React = require('react');
var _ = require('lodash');

var _require = require('antd'),
    Tooltip = _require.Tooltip,
    Icon = _require.Icon;

var YYClass = require('../base/YYClass');
var PropTypes = YYClass.PropTypes;
var field = YYClass.field;
var TreeUtils = require('../../utils/TreeUtils');
var ColumnConfigUtils = require('./utils/ColumnConfigUtils');
require('./style/titlegroup.less');

function noop() {}

//定义META
var uiMeta = {
    name: 'titlegroup',
    fields: [field('group', undefined, PropTypes.array, false, '多表头分组的配置信息'), field('columns', undefined, PropTypes.object, false, '多表头对应的被合并列的定义信息'), field('onOrderChange', noop, PropTypes.func, false, '多表头排序状态变化时触发')]
};

//处理多表头分组末级叶子节点对应的合并列定义的dataIndex关系，通过refDataIndex属性进行标识。
// function groupRefDataIndex(group,columns){
//     //item(当前节点)、index(同级节点之间的索引) 、pos(父子节点层级和索引)、key(当前节点键值)、siblingPos(兄弟节点间的开始first与结束last标识)、parent(父节点)
//     let lastNodeIndex = 0;//末级叶子节点索引
//     TreeUtils.loopAll(group,function (item, index, pos, key, siblingPos, parent) {
//         if(!item.children){
//             if(lastNodeIndex<columns.length){
//                 item.refDataIndex = columns[lastNodeIndex].dataIndex;
//                 item.refRootDataIndex = columns[0].dataIndex;
//             }
//             lastNodeIndex = lastNodeIndex + 1;
//         }
//     });
// }

var YYTitleGroup = YYClass.create({
    uiMeta: uiMeta,
    // statics:{
    //     groupRefDataIndex:groupRefDataIndex
    // },

    getDefaultProps: function getDefaultProps() {
        return {
            onOrderChange: noop
        };
    },

    getInitialState: function getInitialState() {
        return {
            sortColumn: null, //当前排序的列
            sortOrder: null //当前排序的列的顺序
        };
    },
    setColOrderState: function setColOrderState(sortColumn, sortOrder) {
        this.setState({
            sortColumn: sortColumn,
            sortOrder: sortOrder
        });
    },

    render: function render() {
        var rowColums = [];
        var rows = _.isArray(this.props.group) ? this.props.group : [this.props.group];
        //配置rowSpan、colSpan的值
        this._getGroupSpans(rows, rowColums);
        //console.log('groupColumns--->',groupColumns);
        return this._renderGroupTable(rowColums);
    },
    getColumnKey: function getColumnKey(column, index) {
        return column.key || column.dataIndex || index;
    },
    isSortColumn: function isSortColumn(column) {
        var sortColumn = this.state.sortColumn;

        if (!column || !sortColumn) {
            return false;
        }
        return this.getColumnKey(sortColumn) === this.getColumnKey(column);
    },
    drawSorterUpDownArrow: function drawSorterUpDownArrow(cellTitleView, cellProps) {
        var _this = this;

        var sortOrder = this.state.sortOrder;

        var isSortColumn = this.isSortColumn(cellProps);
        if (isSortColumn) {
            cellProps.className = cellProps.className || '';
            if (sortOrder) {
                cellProps.className += ' ant-table-column-sort';
            }
        }
        var isAscend = isSortColumn && sortOrder === 'ascend';
        var isDescend = isSortColumn && sortOrder === 'descend';
        cellTitleView.push(React.createElement(
            'div',
            { className: 'ant-table-column-sorter' },
            React.createElement(
                'span',
                { className: 'ant-table-column-sorter-up ' + (isAscend ? 'on' : 'off'),
                    title: '\u2191',
                    onClick: function onClick() {
                        return _this.toggleSortOrder('ascend', cellProps);
                    }
                },
                React.createElement(Icon, { type: 'caret-up' })
            ),
            React.createElement(
                'span',
                { className: 'ant-table-column-sorter-down ' + (isDescend ? 'on' : 'off'),
                    title: '\u2193',
                    onClick: function onClick() {
                        return _this.toggleSortOrder('descend', cellProps);
                    }
                },
                React.createElement(Icon, { type: 'caret-down' })
            )
        ));
    },
    drawToolTipIcon: function drawToolTipIcon(cellTitleView, cellProps) {
        if (cellProps.titleTip && cellProps.children === undefined) cellTitleView.push(React.createElement(
            Tooltip,
            { title: cellProps.titleTip },
            React.createElement(Icon, { type: 'question-circle-o' })
        ));
    },
    sorterFun: function sorterFun(cellProps) {
        cellProps['sorter'] = ColumnConfigUtils.defaultSortColumnsFun(cellProps);
    },
    toggleSortOrder: function toggleSortOrder(order, column) {
        var _state = this.state,
            sortColumn = _state.sortColumn,
            sortOrder = _state.sortOrder;
        // 只同时允许一列进行排序，否则会导致排序顺序的逻辑问题

        var isSortColumn = this.isSortColumn(column);
        if (!isSortColumn) {
            // 当前列未排序
            sortOrder = order;
            sortColumn = column;
        } else {
            // 当前列已排序
            if (sortOrder === order) {
                // 切换为未排序状态
                sortOrder = '';
                sortColumn = null;
            } else {
                // 切换为排序状态
                sortOrder = order;
            }
        }
        var newState = {
            sortOrder: sortOrder,
            sortColumn: sortColumn
        };
        this.setState(newState);
        this.props.onOrderChange(newState);
    },
    _renderGroupTable: function _renderGroupTable(rows) {
        var _this2 = this;

        var columns = this.props.columns;


        var allCol = [];
        var allColStyles = {};
        //按行遍历多表头
        var allTr = rows.map(function (row, index) {
            var thViews = row.map(function (cellProps, cellIndex) {
                var colStyle = null;
                if (cellProps.refDataIndex && cellProps.width && cellProps.width > 0) {
                    colStyle = { width: cellProps.width, minWidth: cellProps.width, maxWidth: cellProps.width };
                    allColStyles[cellProps.refDataIndex] = colStyle;
                }
                var cellClassName = [];
                //多表头最后节点的表头标题必填样式，通过找到refDataIndex对应的列定义信息进行处理。
                if (cellProps.refDataIndex) {
                    cellClassName.push('yyui-table-groupcol-' + cellProps.refDataIndex); //列标识样式名，拖拽时需要用到。
                    var columnInfo = _.find(columns, ['dataIndex', cellProps.refDataIndex]);
                    if (columnInfo && columnInfo.required) {
                        cellClassName.push('text-required');
                    }
                    if (columnInfo && columnInfo.sorterEnable) {
                        cellProps['sorterEnable'] = columnInfo.sorterEnable;
                    }
                    if (columnInfo && columnInfo.sorter) {
                        cellProps['sorter'] = columnInfo.sorter;
                    }
                    if (columnInfo && columnInfo.dataIndex) {
                        cellProps['dataIndex'] = columnInfo.dataIndex;
                    }
                    if (columnInfo && columnInfo.titleTip) {
                        cellProps['titleTip'] = columnInfo.titleTip;
                    }
                } else {
                    cellClassName.push('yyui-table-anygroupcol'); //非末级标题列样式名标识，拖拽时需要用到。
                }
                //确保过长文本提示悬浮信息
                var cellTitleView = [];
                cellTitleView.push(React.createElement('span', { title: cellProps.title }, cellProps.title));
                _this2.drawToolTipIcon(cellTitleView, cellProps);
                //排序箭头
                if (cellProps.children === undefined && cellProps.sorterEnable) {
                    _this2.drawSorterUpDownArrow(cellTitleView, cellProps);
                    if (cellProps.sorter === undefined) _this2.sorterFun(cellProps);
                }
                return React.createElement(
                    'th',
                    { title: cellProps.title, 'data-refrootdataindex': cellProps.refRootDataIndex, 'data-refdataindex': cellProps.refDataIndex, colSpan: cellProps.colSpan, className: cellClassName.length ? cellClassName.join(' ') : null, rowSpan: cellProps.rowSpan, style: colStyle },
                    cellTitleView
                );
            });
            return React.createElement(
                'tr',
                { key: index },
                thViews
            );
        });
        //依据列定义顺序渲染多表头末级节点的表头宽度
        _.forEach(columns || [], function (column) {
            allCol.push(React.createElement('col', { style: allColStyles[column.dataIndex] }));
        });
        var allColgroup = React.createElement(
            'colgroup',
            null,
            allCol
        );
        return React.createElement(
            'table',
            { className: this.className() },
            allCol.length > 0 ? allColgroup : null,
            allTr
        );
    },
    _getGroupSpans: function _getGroupSpans(allColumns, allRows) {
        //计算分组表头的rowSpan和colSpan值
        var _groupColumns = function _groupColumns(columns) {
            var currentRow = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
            var parentColumn = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
            var rows = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : [];

            // track how many rows we got
            rows[currentRow] = rows[currentRow] || [];
            var grouped = [];
            var setRowSpan = function setRowSpan(column) {
                var rowSpan = rows.length - currentRow;
                if (column && !column.children && rowSpan > 1 && ( // parent columns are supposed to be one row
                !column.rowSpan || column.rowSpan < rowSpan)) {
                    column.rowSpan = rowSpan;
                }
            };
            _.forEach(columns, function (column, index) {
                var newColumn = _.assign({}, column);
                rows[currentRow].push(newColumn);
                parentColumn.colSpan = parentColumn.colSpan || 0;
                if (newColumn.children && newColumn.children.length > 0) {
                    newColumn.children = _groupColumns(newColumn.children, currentRow + 1, newColumn, rows);
                    parentColumn.colSpan = parentColumn.colSpan + newColumn.colSpan;
                } else {
                    parentColumn.colSpan++;
                }
                // update rowspan to all same row columns
                for (var i = 0; i < rows[currentRow].length - 1; ++i) {
                    setRowSpan(rows[currentRow][i]);
                }
                // last column, update rowspan immediately
                if (index + 1 === columns.length) {
                    setRowSpan(newColumn);
                }
                grouped.push(newColumn);
            });
            return grouped;
        };
        var groupColumns = _groupColumns(allColumns, 0, {}, allRows);
        return groupColumns;
    }
});

module.exports = YYTitleGroup;