'use strict';

/**
 * 表格列配置工具
 * @type {{}}
 */
var _ = require('lodash');
var assign = require('lodash/assign');
var YYMessage = require('../../message/YYMessage');
var DataUtil = require('../../../utils/DataUtil');
var AuthToken = require('../../../utils/AuthToken');
var TreeUtils = require('../../../utils/TreeUtils');
// var YYTitleGroup = require('../YYTitleGroup');
var ColumnConfigUtils = {};
// 获取表格列配置缓存键值
function getColumnConfigKey(options) {
    var _ref = options || {},
        cookieKey = _ref.cookieKey,
        pageCode = _ref.pageCode,
        tableKey = _ref.tableKey;

    var userId = AuthToken.getUserId();
    if (cookieKey) {
        return cookieKey + "_" + userId;
    } else if (pageCode && tableKey) {
        return pageCode + "_" + tableKey + "_" + userId;
    }
    return null;
}
// 获取表格滚动高度缓存键值
function getTableScrollKey(options) {
    var _ref2 = options || {},
        cookieKey = _ref2.cookieKey,
        pageCode = _ref2.pageCode,
        tableKey = _ref2.tableKey;

    var userId = AuthToken.getUserId();
    if (cookieKey) {
        return cookieKey + "_scroll_" + userId;
    } else if (pageCode && tableKey) {
        return pageCode + "_" + tableKey + "_scroll_" + userId;
    }
    return null;
}

function dealGroupColumns(columns, cachedConfigs) {
    // 对比设计器和缓存中的多表头配置
    ColumnConfigUtils.initGroupColumnInfo(columns);
    var groupColumns = _.filter(columns, function (column) {
        return isTitleGroup(column);
    });
    var cachedGroupColumns = _.filter(cachedConfigs, function (cachedColumn) {
        return isTitleGroup(cachedColumn);
    });
    var replaceIndexs = [];
    // 设计器中的多表头配置
    _.forEach(groupColumns, function (column) {
        var cachedColumn = _.find(cachedGroupColumns, ['dataIndex', column.uikey]);
        var changed = false;
        if (!cachedColumn) {
            changed = true;
        } else {
            _.forEach(column.titleGroup, function (group) {
                var cachedGroup = TreeUtils.findWithPropName(cachedColumn.titleGroup, 'nid', group.nid, true, true);
                if (!cachedGroup || _.isEmpty(cachedGroup)) {
                    changed = true;
                    return false;
                } else {
                    _.forEach(cachedGroup, function (item) {
                        if (item.version !== group.version) {
                            // 比较version是否一致
                            changed = true;
                            return false;
                        }
                    });
                }
            });
        }
        if (changed) {
            TreeUtils.loopTreeData(column.titleGroup, function (item, index, arr) {
                if (item.refDataIndex) {
                    replaceIndexs.push(item.refDataIndex);
                }
            });
        }
    });
    // 缓存中的多表头配置
    _.forEach(cachedGroupColumns, function (cachedColumn) {
        var column = _.find(groupColumns, ['uikey', cachedColumn.dataIndex]);
        if (!column) {
            TreeUtils.loopTreeData(cachedColumn.titleGroup, function (item, index, arr) {
                if (item.refDataIndex) {
                    replaceIndexs.push(item.refDataIndex);
                }
            });
        }
    });
    if (replaceIndexs && replaceIndexs.length) {
        _.forEach(replaceIndexs, function (key) {
            var _cachedIndex = _.findIndex(cachedConfigs, ['dataIndex', key]);
            var _index = _.findIndex(columns, ['uikey', key]);
            if (_cachedIndex !== -1) {
                if (_index === -1) {
                    cachedConfigs.splice(_index, 1);
                } else if (columns[_index].titleGroup) {
                    cachedConfigs[_cachedIndex].titleGroup = columns[_index].titleGroup;
                } else {
                    delete cachedConfigs[_cachedIndex].titleGroup;
                }
            }
        });
    }
    return cachedConfigs;
}

// 判断是否合并列
function isTitleGroup(column) {
    if (!column) {
        return false;
    }
    return column.titleGroup && column.titleGroup.length;
}

//处理多表头分组末级叶子节点对应的合并列定义的dataIndex关系，通过refDataIndex属性进行标识。
ColumnConfigUtils.groupRefDataIndex = function groupRefDataIndex(group, columns) {
    //item(当前节点)、index(同级节点之间的索引) 、pos(父子节点层级和索引)、key(当前节点键值)、siblingPos(兄弟节点间的开始first与结束last标识)、parent(父节点)
    var lastNodeIndex = 0; //末级叶子节点索引
    TreeUtils.loopAll(group, function (item, index, pos, key, siblingPos, parent) {
        if (!item.children) {
            if (lastNodeIndex < columns.length) {
                item.refDataIndex = columns[lastNodeIndex].dataIndex;
                item.refRootDataIndex = columns[0].dataIndex;
            }
            lastNodeIndex = lastNodeIndex + 1;
        }
    });
};

/**
 * 结合缓存的表格列配置, 返回合并处理后的表格列
 * @param options
 *          columns 原表格列
 *          pageCode 页面编号
 *          tableKey 表格键值
 *          cookieKey 表格缓存键值
 */
ColumnConfigUtils.combineColumnConfigs = function (options) {
    var _ref3 = options || {},
        columns = _ref3.columns;

    var cachedConfigs = this.getCachedColumnConfigs(options);
    if (!cachedConfigs) {
        return columns;
    }
    if (!_.isPlainObject(cachedConfigs)) {
        cachedConfigs = JSON.parse(cachedConfigs);
    }
    cachedConfigs = dealGroupColumns(columns, cachedConfigs);
    var targetColumns = [];
    if (_.isArray(cachedConfigs) && cachedConfigs.length > 0) {
        var matchedColumns = [];
        _.forEach(cachedConfigs, function (cachedItem) {
            _.forEach(columns, function (originItem) {
                if (cachedItem.dataIndex === originItem.dataIndex && originItem.isShow != false) {
                    var retItem = assign({}, originItem, cachedItem);
                    matchedColumns.push(retItem);
                    return false;
                }
            });
        });
        var notMatchedColumns = [];
        _.forEach(columns, function (originItem) {
            var repeat = false;
            _.forEach(matchedColumns, function (matchedItem) {
                if (originItem.dataIndex === matchedItem.dataIndex) {
                    repeat = true;
                    return false;
                }
            });
            if (!repeat) {
                notMatchedColumns.push(originItem);
            }
        });
        targetColumns = matchedColumns.concat(notMatchedColumns);
    }
    return targetColumns;
};

// 初始化多表头的自定义列配置
ColumnConfigUtils.initGroupColumnInfo = function (columns) {
    for (var i = 0; i < columns.length; i++) {
        var col = columns[i];
        if (col.colSpan > 0 && !_.isEmpty(col.titleGroup)) {
            //处理分组标题
            //获取被合并的可见列信息
            var groupColumns = [];
            var groupColumnsSpan = 0;
            for (var colSpanIdx = i; colSpanIdx < columns.length; colSpanIdx++) {
                var colSpanColumn = columns[colSpanIdx];
                if (colSpanColumn) {
                    groupColumns.push(colSpanColumn);
                    groupColumnsSpan++;
                }
                if (groupColumnsSpan == col.colSpan) break;
            }
            //构建多表头末级节点与数据列定义之间的映射关系refDataIndex
            this.groupRefDataIndex(col.titleGroup, groupColumns);
        }
    }
};
/**
 * 结合缓存的表格列配置, 返回合并处理后的表格列
 * @param options
 *          columns 原表格列
 *          pageCode 页面编号
 *          tableKey 表格键值
 *          cookieKey 表格缓存键值
 */
ColumnConfigUtils.showColumnConfig = function (options) {
    var _ref4 = options || {},
        columns = _ref4.columns,
        RowNum = _ref4.RowNum;

    var cachedColumns = this.getCachedColumnConfigs(options);
    if (cachedColumns) {
        if (!_.isPlainObject(cachedColumns)) {
            cachedColumns = JSON.parse(cachedColumns);
        }
        cachedColumns = dealGroupColumns(columns, cachedColumns);
        var matchedColumns = [];
        _.forEach(cachedColumns, function (cachedItem) {
            _.forEach(columns, function (originItem) {
                if (cachedItem.dataIndex === originItem.dataIndex && originItem.isShow != false) {
                    var retItem = assign({}, originItem, cachedItem);
                    retItem.fixed = cachedItem.fixed;
                    matchedColumns.push(retItem);
                    return false;
                }
            });
        });
        var notMatchedColumns = [];
        _.forEach(columns, function (originItem) {
            var repeat = false;
            _.forEach(matchedColumns, function (matchedItem) {
                if (originItem.dataIndex === matchedItem.dataIndex) {
                    repeat = true;
                    return false;
                }
            });
            if (!repeat && originItem.isShow != false && originItem.dataIndex !== "operate" && originItem.dataIndex !== RowNum) {
                notMatchedColumns.push(originItem);
            }
        });
        columns = matchedColumns.concat(notMatchedColumns);
    } else {
        // 过滤掉设计器中配置为不显示的列,操作列,序号列
        columns = _.filter(columns, function (column) {
            return column && column.isShow != false && column.dataIndex !== "operate" && column.dataIndex !== RowNum;
        });
    }
    return this.groupByFixed(columns);
};

// 按冻结列配置进行分组
ColumnConfigUtils.groupByFixed = function (columns) {
    if (!columns || !_.isArray(columns) || columns.length == 0) {
        return columns;
    }
    var leftFixedColumns = [],
        rightFixedColumns = [],
        notFixedColumns = [];
    _.forEach(columns, function (col) {
        switch (col.fixed) {
            case 'left':
                leftFixedColumns.push(col);
                break;
            case 'right':
                rightFixedColumns.push(col);
                break;
            default:
                notFixedColumns.push(col);
                break;
        }
    });
    return leftFixedColumns.concat(notFixedColumns).concat(rightFixedColumns);
};

/**
 * 表格列配置完毕
 * @param options
 *          columns 原表格列
 *          columnConfigs 待缓存的表格列配置
 *          pageCode 页面编号
 *          tableKey 表格键值
 *          scroll 表格高度配置
 *          cookieKey 表格缓存键值
 */
ColumnConfigUtils.columnConfigDone = function (options) {
    var _ref5 = options || {},
        columns = _ref5.columns,
        columnConfigs = _ref5.columnConfigs,
        RowNum = _ref5.RowNum;
    // 1. 删除原有的缓存数据


    this.delCachedColumnConfigs(options);
    // 2. 组装待保存的数据,过滤掉操作列和序号列
    columnConfigs = columnConfigs.filter(function (column) {
        return column && column.dataIndex !== "operate" && column.dataIndex !== RowNum;
    }) || [];
    var dataToCache = _.cloneDeep(columnConfigs).map(function (item) {
        return {
            dataIndex: item.dataIndex,
            fixed: item.fixed,
            isShow: item.isShow,
            width: item.width,
            titleGroup: item.titleGroup
        };
    });
    // 3.保存数据到缓存
    options.data = dataToCache;
    var success = this.saveColumnConfigs(options);
    if (!success) {
        return columns;
    }
    // 4.结合缓存进行展示
    var matchedColumns = [];
    _.forEach(dataToCache, function (cachedItem) {
        _.forEach(columns, function (originItem) {
            if (cachedItem.dataIndex === originItem.dataIndex && originItem.isShow != false) {
                var retItem = assign({}, originItem, cachedItem);
                matchedColumns.push(retItem);
                return false;
            }
        });
    });
    var notMatchedColumns = [];
    _.forEach(columns, function (originItem) {
        var repeat = false;
        _.forEach(matchedColumns, function (matchedItem) {
            if (originItem.dataIndex === matchedItem.dataIndex) {
                repeat = true;
                return false;
            }
        });
        if (!repeat) {
            notMatchedColumns.push(originItem);
        }
    });
    return matchedColumns.concat(notMatchedColumns);
};

/**
 * 从浏览器Local Storage中获取表格列配置
 * @param options
 *          pageCode 页面编号
 *          tableKey 表格键值
 *          cookieKey 表格缓存键值
 * @returns {*}
 */
ColumnConfigUtils.getCachedColumnConfigs = function (options) {
    var key = getColumnConfigKey(options);
    if (!key) {
        return null;
    }
    return DataUtil.localRead(key);
};

/**
 * 获取缓存的表格高度配置
 * @param options
 *          pageCode 页面编号
 *          tableKey 表格键值
 *          cookieKey 表格缓存键值
 * @returns {*}
 */
ColumnConfigUtils.getCachedTableScroll = function (options) {
    var key = getTableScrollKey(options);
    if (!key) {
        return null;
    }
    var data = DataUtil.localRead(key);
    if (!_.isPlainObject(data)) {
        data = JSON.parse(data);
    }
    return data;
};

/**
 * 保存表格配置列数据到Local Storage
 * @param options
 *          pageCode 页面编号
 *          tableKey 表格键值
 *          cookieKey 表格缓存键值
 *          data 表格列配置数据
 *          scroll 表格高度配置数据
 */
ColumnConfigUtils.saveColumnConfigs = function (options) {
    var columnKey = getColumnConfigKey(options);
    var scrollKey = getTableScrollKey(options);

    var _ref6 = options || {},
        data = _ref6.data,
        scroll = _ref6.scroll;

    if (!columnKey || !scrollKey) {
        YYMessage.error("表格自定义列配置保存失败!", 3);
        console.error('表格自定义列配置保存失败', { cookieKey: cookieKey, pageCode: pageCode, tableKey: tableKey, userId: userId });
        return false;
    }
    if (data && !_.isEmpty(data)) {
        DataUtil.localSave(columnKey, data);
    }
    if (scroll && !_.isEmpty(scroll)) {
        DataUtil.localSave(scrollKey, scroll);
    }
    return true;
};

/**
 * 表格拖拽后更新缓存的列配置信息
 * @param options
 *          columns 新的表格列定义
 *          cookieKey 表格缓存键值
 *          pageCode 页面编号
 *          tableKey 表格键值
 */
ColumnConfigUtils.cacheAfterResize = function (options) {
    var key = getColumnConfigKey(options);
    if (!key) {
        return;
    }
    // 每次拖拽之后都重新缓存
    this.delCachedColumnConfigs(options);

    var _ref7 = options || {},
        columns = _ref7.columns;

    var isTitleGroup = _.filter(columns, function (column) {
        return column.titleGroup && column.titleGroup.length;
    });
    var mixedColumns = this.showColumnConfig(options);
    if (isTitleGroup) {
        this.initGroupColumnInfo(mixedColumns);
    }
    this.columnConfigDone(assign({}, options, { columnConfigs: mixedColumns }));
};

/**
 * 从浏览器Local Storage中删除表格列配置
 * @param options
 *          pageCode 页面编号
 *          tableKey 表格键值
 *          cookieKey 表格缓存键值
 */
ColumnConfigUtils.delCachedColumnConfigs = function (options) {
    var columnKey = getColumnConfigKey(options);
    var scrollKey = getTableScrollKey(options);
    DataUtil.localRemove([columnKey, scrollKey]);
};

/**
 * 预制排序函数
 * @param col
 * @returns {Function}
 */
ColumnConfigUtils.defaultSortColumnsFun = function (col) {
    return function (a, b) {
        if (col.fieldtype === 'number' || col.fieldtype === 'currency' || _.isNumber(a[col.dataIndex]) && _.isNumber(b[col.dataIndex])) return a[col.dataIndex] - b[col.dataIndex];
        if (col.fieldtype === 'string' || _.isString(a[col.dataIndex]) && _.isString(b[col.dataIndex])) return a[col.dataIndex].localeCompare(b[col.dataIndex]);
        if (col.fieldtype === 'date' || _.isDate(a[col.dataIndex]) && _.isDate(a[col.dataIndex])) return new Date(a[col.dataIndex]) - new Date(b[col.dataIndex]);
    };
};

module.exports = ColumnConfigUtils;